@@ -1,11 +1,11 @@
 /* Save the list of files in a directory in a string.
 
-   Copyright 1997, 1999, 2001, 2003 Free Software Foundation, Inc.
+   Copyright (C) 1997, 1999, 2001, 2003, 2005 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation; either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,14 +13,14 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 /* Written by David MacKenzie <djm@gnu.ai.mit.edu>. */
 
 #if !defined SAVEDIR_H_
 # define SAVEDIR_H_
 
-char *savedir (const char *dir);
+char *savedir (char const *dir);
+char *fdsavedir (int fd);
 
 #endif
